'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var Prism = _interopDefault(require('prismjs'));
var React = require('react');
var immutable = require('immutable');

const TOKEN_MARK = 'prism-token';

/**
 * Default filter for code blocks
 */
function defaultOnlyIn(node) {
  return node.object === 'block' && node.type === 'code_block';
}
/**
 * Default getter for syntax
 */


function defaultGetSyntax(node) {
  return 'javascript';
}
/**
 * Default rendering for marks
 */


function defaultRenderMark(props) {
  const mark = props.mark;

  if (mark.type !== TOKEN_MARK) {
    return undefined;
  }

  const className = mark.data.get('className');
  return React.createElement("span", {
    className: className
  }, props.children);
}
/**
 * The plugin options
 */


class Options extends immutable.Record({
  onlyIn: defaultOnlyIn,
  getSyntax: defaultGetSyntax,
  renderMark: defaultRenderMark
}) {}

/**
 * A Slate plugin to highlight code syntax.
 */

function PrismPlugin(optsParam) {
  if (optsParam === void 0) {
    optsParam = {};
  }

  const opts = new Options(optsParam);
  return {
    decorateNode: node => {
      if (!opts.onlyIn(node)) {
        return undefined;
      }

      return _decorateNode(opts, node);
    },
    renderMark: opts.renderMark,
    TOKEN_MARK
  };
}
/**
 * Returns the decoration for a node
 */


function _decorateNode(opts, block) {
  const grammarName = opts.getSyntax(block);
  const grammar = Prism.languages[grammarName];

  if (!grammar) {
    // Grammar not loaded
    return [];
  } // Tokenize the whole block text


  const texts = block.getTexts();
  const blockText = texts.map(t => t.text).join('\n');
  const tokens = Prism.tokenize(blockText, grammar); // The list of decorations to return

  const decorations = [];
  let textStart = 0;
  let textEnd = 0;
  texts.forEach(text => {
    textEnd = textStart + text.text.length;
    let offset = 0;

    function processToken(token, accu) {
      accu = accu || '';

      if (typeof token === 'string') {
        if (accu) {
          const decoration = createDecoration({
            text,
            textStart,
            textEnd,
            start: offset,
            end: offset + token.length,
            className: `prism-token token ${accu}`
          });

          if (decoration) {
            decorations.push(decoration);
          }
        }

        offset += token.length;
      } else {
        accu = `${accu} ${token.type} ${token.alias || ''}`;

        if (typeof token.content === 'string') {
          const decoration = createDecoration({
            text,
            textStart,
            textEnd,
            start: offset,
            end: offset + token.content.length,
            className: `prism-token token ${accu}`
          });

          if (decoration) {
            decorations.push(decoration);
          }

          offset += token.content.length;
        } else {
          // When using token.content instead of token.matchedStr, token can be deep
          for (let i = 0; i < token.content.length; i += 1) {
            processToken(token.content[i], accu);
          }
        }
      }
    }

    tokens.forEach(processToken);
    textStart = textEnd + 1; // account for added `\n`
  });
  return decorations;
}
/**
 * Return a decoration range for the given text.
 */


function createDecoration(_ref) {
  let text = _ref.text,
      textStart = _ref.textStart,
      textEnd = _ref.textEnd,
      start = _ref.start,
      end = _ref.end,
      className = _ref.className;

  if (start >= textEnd || end <= textStart) {
    // Ignore, the token is not in the text
    return null;
  } // Shrink to this text boundaries


  start = Math.max(start, textStart);
  end = Math.min(end, textEnd); // Now shift offsets to be relative to this text

  start -= textStart;
  end -= textStart;
  return {
    anchorKey: text.key,
    anchorOffset: start,
    focusKey: text.key,
    focusOffset: end,
    marks: [{
      type: 'prism-token',
      data: {
        className
      }
    }]
  };
}

exports.default = PrismPlugin;
//# sourceMappingURL=slate-prism.js.map
